/* SpotWelder Steuerung
 * Harald Sattler, 2019-06-29
 *
 * Der Attiny85 im DigiSpark laeuft mit 16,5 Mhz
 *
 * Basisplatine mit Anschluessen fuer 
 * - Fusspedal zum Ausloesen des Schweissimpulses
 * - Spannungsversorgung (DC, max. 14V. Der Spanungsregler des DigiSpark versorgt auch den Rest der Schaltung)
 * - SolidState Relais das den Mikrowellen-Trafo schaltet
 * - Stecksockel fuer den DigiSpark MicroController
 * - Stecksockel fuer das OLED Display (1,6" Diagonale, 128 x 64 Pixel)
 * Wichtig!
 * Der hohe Stromimpuls des Trafo bewirkt, dass das Einschaltsignal vom Microcontroller an das SolidState Relais fruehzeitig
 *   abgebrochen wird, die eingestellte Zeit wird also nicht erreicht.
 *   Daher ist auf der Basisplatine ein Stuetzkondensator platziert, der diesen Effekt unterdrckt.
 *
 * Mit einem Spindelpoti wird die Impulsdauer eingestellt.
 * Ein Microtaster parallel zum Anschluss fuer den Fussschalter loest den Impuls ebenfalls aus (Test).
 * Der Impuls wird nur einmal ausgeloest, der Taster/Fussschalter muss losgelassen werden, bevor erneut ausgeloest werden kann.
 *
 * Der Taster/Fusspedal bentigt einen externen PullUp mit maximal 1,5 kOhm, da am selben Pin die Beschaltung fr den USB haengt.
 *   Die Z-Diode zieht ueber einen 68 Ohm Widerstand nach Masse. Mit PullUp von 1,5 kOhm erreicht der Pin einen Ruhepegel von
 *   ~3,2 V, was als High erkannt wird.
 * 
 * Mit dem Poti wird die Impulsdauer eingestellt und auf dem OLED-Display angezeigt.
 * Das Display wird nur aktualisiert, wenn sich die Einstellung des Poti geaendert hat.
 * Mit dem Taster/Fusspedal wird der Impuls ausgeloest.
 * Am Ausgang "Relais" wird ein SolidState Relais angesteuert, das den umgebauten Mikrowellen-Trafo primaerseitig fuer die 
 * Dauer des Impulses einschaltet. Moechte man ein normales Relais verwenden, muss eine passende Treiberstufe zwischengeschaltet
 * werden.
 * 
 * Das Display zeigt die minimal und maximal einstellbare sowie die aktuell eingestellte Impulsdauer an.
 * Fuer die Dauer des Impulses wird - als visuelle Rueckmeldung - das Display invers geschaltet (bei (zu) kurzer Impulsdauer wird 
 *   nur ein schmaler Balken invers dargestellt).
 *
 * 2019-07-12, V2.0
 * - Implementierung eines Menues zur Einstellung der Impulsanzahl
 *   Wird beim Einschalten (und nach Ablauf der Bootloader-Zeit) der Taster gedrueckt gehalten, wird direkt das Einstellungsmenue aufgerufen.
 *   Im laufenden Betrieb kann durch Festhalten des Taster fuer mehr als 1 Sekunde ebenfalls das Menue aufgerufen werden.
 * (- Speicherung der Einstellungen im EEPROM - (noch?) nicht implementiert, kein Platz mehr)
 * - Implementierung von Mehrfachimpulsen - die zur Laufzeit eingestellte Anzahl von Impulsen wird ausgegeben.
 *   Nach Reboot ist die Anzahl immer 1.
 */
 
// includes
#include <Wire.h>
#include <EEPROM.h>

 // defines fuer die Pins
 #define TASTER 4
 #define POTI 	3
 #define RELAIS 1
 #define LED 5											// nicht verwenden, der DigiSpark wird angehalten, wenn an Pin 5 eine LED (oder ein PullUp!) angeschlossen ist
 
 // definitions IIC & OLED
#define PIN_I2C_SDA 0									// TX-Pin - die Pins koennen nicht geaendert werden!! Ist in der Lib so festgelegt
#define PIN_I2C_SCL 2									// RX-Pin - die Pins koennen nicht geaendert werden!! Ist in der Lib so festgelegt
#define OLED_address  0x3c							    // I2C-Adresse des OLED Display

// defines fuer EEPROM Zugriff
#define EEPARAMS 200									// Startadresse fuer Parameterspeicher

// globale Variablen
 unsigned int potiWert;									// per analogRead() gelesener Wert des Poti
 unsigned long weldingTime;								// Wert des Impulses in [ms]
 //unsigned long weldingMicros;
 unsigned long lastWeldingTime = 0;						// Merker fuer ScreenUpdate (nur aktualisieren, wenn Aenderung des Wertes)
 bool tasterSet = false;								// Flag fuer "ist der Taster gedrueckt?"
 unsigned long tasteLang = 0;							// Merker fuer langen Tastendruck

 // Puffer fuer OLED String-Ausgabe
char OLEDBuffer[34];
static char ZeilenOffset = 2;							// linker Rand fuer Textausgaben auf OLED

const unsigned int minImpulsAllowed = 10;				// minimale Impulsdauer
const unsigned int maxImpulsAllowed = 300;				// maximale Impulsdauer

unsigned int impulsAnzahl = 1;							// Default fuer Impulsanzahl, kann per Menue geandert werden
unsigned int impulsDauer = 20;							// Default fuer Impulsdauer, kann per Menue geandert werden
bool inEinstellungen = false;							// Flag "wir sind in Einstellungen"
unsigned char menueState = 0;							// welcher Parameter soll geaendert werden


void readPoti()
{
	// Poti auslesen, ergibt Wert 0..1023
	potiWert = analogRead(POTI);

	// daraus Zeit in ms in weldingTime ablegen
	weldingTime = map(potiWert, 0, 1023, minImpulsAllowed, maxImpulsAllowed);	// eingrenzen auf erlaubte min/max Werte
	//weldingMicros = weldingTime * 1000;					// Probleme zur  Laufzeit mit delay(), daher delayMicroseconds() verwendet
}

void Einstellungen()
{
	// Ausgabe des Menues
	clear_display();
	sendStrXY("Einstellungen", 0, 2);						// 16 chars / line

	sendStrXY("Impulse:", 2, 0);
	//sendStrXY("Dauer [ms]:", 4, 0);

	menueState = 1;											// initial immer Impulsanzahl auswaehlen

	// Poti abfragen und Wert merken fuer Vergleich auf Aenderung
	unsigned int potiNav = analogRead(POTI);
	unsigned int potiNavWert = 0;

	while (inEinstellungen == true)
	{
		// Abfrage des Tasters zur Navigation im Menue
		if ((digitalRead(TASTER) == 0) && (tasterSet == false))	// Taster (nach offen) gedrueckt
		{
			// feststellen ob Taste lange (> 1s) gedrueckt wird
			tasteLang = millis();
			tasterSet = true;

			// Behandlung der Tastendruecke
			//if (menueState == 1)							// Impulsanzahl wird eingestellt
			//{
			//	menueState = 2;
			//}
			//else 
			//{
			//	menueState = 1;
			//}
		}
		else if ((tasterSet == true) && (digitalRead(TASTER) == 1))	// Taster wieder losgelassen
		{
			tasteLang = millis();								// Status "Taste lange gedrueckt" loeschen
			tasterSet = false;
		}
		else if ((digitalRead(TASTER) == 0) && (tasterSet == true))	// Taster festgehalten
		{
			if (millis() > tasteLang + 1000)					// Taste laenger als 1 Sekunde gedrueckt
			{
				tasteLang = millis();
				inEinstellungen = false;						// Einstellungen verlassen
				clear_display();								// Display loeschen
				lastWeldingTime = 0;							// Screen refresh erzwingen
			}
			delay(100);											// bei gedrueckter Taste Prellen unterdruecken
		}

		// Markierung des aenderbaren Parameters mit "x" in letzter Spalte
		//sendStrXY("x", 2 * menueState, 15);
		// Loeschen des jeweils anderen "x"
		//sendStrXY(" ", 6 - (2 * menueState), 15);

		// Poti fuer Vergleich einlesen
		potiNavWert = analogRead(POTI);

		// Abfrage des Poti zum einstellen der Parameter
		if ((potiNav - potiNavWert) > 10)						// Parameter erhoehen
		{
			//if (menueState == 1)
			//{
				impulsAnzahl++;
			//}
			//else
			//{
			//	impulsDauer++;
			//}
			potiNav = potiNavWert;
		}

		if ((potiNavWert - potiNav) > 10)						// Parameter verringern
		{
			//if (menueState == 1)
			//{
				impulsAnzahl--;
			//}
			//else
			//{
			//	impulsDauer--;
			//}
			potiNav = potiNavWert;
		}

		// Parameter auf zulaessige Werte beschraenken
		impulsAnzahl = constrain(impulsAnzahl, 1, 5);
		//impulsDauer = constrain(impulsDauer, minImpulsAllowed, maxImpulsAllowed);

		// Parameter anzeigen
		dtostrf(impulsAnzahl, 2, 0, OLEDBuffer);				// Impuls Anzahl in String wandeln
		sendStrXY(OLEDBuffer, 2, 11);
		//dtostrf(impulsDauer, 2, 0, OLEDBuffer);					// Impuls Dauer in String wandeln
		//sendStrXY(OLEDBuffer, 4, 11);

		// Speicherung der Einstellungen im EEPROM


		// debug: Status "Einstellungen"
		//sendStrXY("e", 0, 0);
	}
}

void setup()
{
	// Pins initialisieren
	pinMode(TASTER, INPUT_PULLUP);
	pinMode(RELAIS, OUTPUT);
	pinMode(LED, INPUT);

	// nicht verwenden! (siehe oben)
	//for (unsigned char i = 0; i < 1; i++) { digitalWrite(RELAIS, 1); delay(500); digitalWrite(RELAIS, 0); delay(500);}
	// die OnBoard LED auch nicht, sonst wird der Trafo fr die Dauer des Blinkens eingschaltet (der zieht Einiges an Strom)
	//for (unsigned char i = 0; i < 10; i++) { digitalWrite(RELAIS, 1); delayMicroseconds(1000 * 1000); digitalWrite(RELAIS, 0); delayMicroseconds(1000 * 1000);}

	// Initialisierung IIC OLED Dispaly	
	//Wire.pins(PIN_I2C_SDA, PIN_I2C_SCL);
	Wire.begin();

	// Serial.println("Initialise OLED screen...");
	StartUp_OLED();

	// Taster gedrueckt? -> Ja, dann ins Einstellungen-Menue
	if (digitalRead(TASTER) == 0)
	{
		inEinstellungen = true;								// Flag setzen und ...
		Einstellungen();									// ...Einstellungen aufrufen
	}
	// Ausgabe des Splashscreen
	clear_display();
	sendStrXY("SpotWelder", 0, 2);							// 16 chars / line
	sendStrXY("V2.0 - 07/2019", 2, 1);
	// sendStrXY("  ", 4, ZeilenOffset);
	sendStrXY("Harald Sattler", 6, 1);
	delay(2000);
	clear_display();
}

void loop()
{
	static unsigned long looptime = 0;

	// debug: Status "loop"
	//sendStrXY("l", 0, 0);

	// Ausgabe auf OLED, jede Sekunde aktualisiert
	if (millis() > looptime + 500) 							// jede halbe Sekunde aktualisieren
	{	
		readPoti();											// liest Poti und liefert Impulsdauer in ms
		
		if (lastWeldingTime != weldingTime)					// hat sich der Wert geaendert? (Poti gedreht, Drift, ...)
		{
			// geanderte Impulszeit merken
			lastWeldingTime = weldingTime; 

			// dann Display aktualisieren
			sendStrXY("SpotWelder", 0, ZeilenOffset);		// 16 chars / line

			// Impulsdauer-Grenzen ausgeben
			sendStrXY("Dauer   ..    ms", 4, 0);
			dtostrf(minImpulsAllowed, 2, 0, OLEDBuffer);	// minimale Impulsdauer in String wandeln
			sendStrXY(OLEDBuffer, 4, 6);
			if (maxImpulsAllowed < 100) {
				dtostrf(maxImpulsAllowed, 2, 0, OLEDBuffer);// maximale Impulsdauer in String wandeln
			}
			else {
				dtostrf(maxImpulsAllowed, 3, 0, OLEDBuffer);// maximale Impulsdauer in String wandeln
			}
			sendStrXY(OLEDBuffer, 4, 10);

			// Temperatur als Int in den OLEDBuffer kopieren
			if (weldingTime < 100) {
				dtostrf(weldingTime, 2, 0, OLEDBuffer);		// aktuelle Impulsdauer in String wandeln
			}
			else {
				dtostrf(weldingTime, 3, 0, OLEDBuffer);		// aktuelle Impulsdauer in String wandeln
			}
			sendStrXY("Impuls:    ms", 6, ZeilenOffset);
			sendStrXY(OLEDBuffer, 6, ZeilenOffset + 8);
		}
		looptime = millis();
	}
	
	// Taster abfragen und Impuls ausloesen (Taster ist aktiv LOW)
	if((digitalRead(TASTER) == 0) && (tasterSet == false))	// Taster (nach offen) gedrueckt
	{
		// feststellen ob Taste lange (> 1s) gedrueckt wird
		tasteLang = millis();

		for (char counter = 0; counter < impulsAnzahl;  counter++) // Anzahl parametrierter Impulse ausgeben
		{
			// Set RelaisPin 
			sendcommand(0xa7);									// Set Inverse Display (visuelle Rueckmeldung)

			digitalWrite(RELAIS, 1);							// Impuls einschalten
			// delayMicroseconds(weldingMicros);					// Impulsdauer abwarten
			delay(weldingTime);									// Impulsdauer abwarten
			digitalWrite(RELAIS, 0);							// Impuls wieder ausschalten

			sendcommand(0xa6);									// Set Normal Display (not inverse) (visuelle Rueckmeldung)
			delay(200);											// kurz warten bevor ein neuer Impuls ausgeloest werden kann (Tastenprellen)
		}

		tasterSet = true;
	}
	else if((tasterSet == true) && (digitalRead(TASTER) == 1))	// Taster wieder losgelassen
	{
		tasteLang = millis();								// Status "Taste lange gedrueckt" loeschen

		tasterSet = false;
	}
	else if ((digitalRead(TASTER) == 0) && (tasterSet == true))	// Taster festgehalten
	{
		if (millis() > tasteLang + 1000)					// Taste laenger als 1 Sekunde gedrueckt
		{
			tasterSet = false;								// Status "Taste gedrueckt" loeschen
			tasteLang = millis();
			inEinstellungen = true;							// Flag setzen und ...
			Einstellungen();								// ...Einstellungen aufrufen
		}
		delay(100);											// bei gedrueckter Taste Prellen unterdruecken
	}
}